/*
 * LCD peripheral support for the JZ4780 and related SoCs.
 *
 * Copyright (C) 2018, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include "lcd-jz4740.h"



/* C++ language interface. */

#ifdef __cplusplus

#include <l4/devices/hw_mmio_register_block.h>

class Lcd_jz4780_chip : public Lcd_jz4740_chip
{
protected:
  /* OSD configure register value calculation. */

  uint32_t _osd_config_irq();

  /* Priority level threshold value calculation. */

  uint32_t _priority_transfer();

  /* Position value calculation. */

  uint32_t _position_bpp();

  /* Descriptor initialisation. */

  void _set_descriptor(struct Jz4740_lcd_descriptor &desc, l4_addr_t source,
                       l4_size_t size, struct Jz4740_lcd_descriptor *next,
                       uint32_t flags = 0, bool frame_enable = true);

public:
  Lcd_jz4780_chip(l4_addr_t addr, Jz4740_lcd_panel *panel);

  /* Configuration. */

  void config(struct Jz4740_lcd_descriptor *desc_vaddr,
              struct Jz4740_lcd_descriptor *desc_paddr,
              l4_addr_t fb_paddr);
};

#endif



/* C language interface mainly provided by jz4740_lcd functions. */

EXTERN_C_BEGIN

void *jz4780_lcd_init(l4_addr_t lcd_base, struct Jz4740_lcd_panel *panel);

EXTERN_C_END
